from sqlalchemy import Column, String
from sqlalchemy.orm import declarative_base

Base = declarative_base()


class Dfkkzp(Base):
    __tablename__ = 'DFKKZP'

    mandt = Column(String, primary_key=True)
    keyz1 = Column(String)
    posza = Column(String)
    selt1 = Column(String)
    selt2 = Column(String)
    selt3 = Column(String)
    selw1 = Column(String)
    selw2 = Column(String)
    selw3 = Column(String)
    betrz = Column(String)
    betrh = Column(String)
    betrk = Column(String)
    betrl = Column(String)
    xtkla = Column(String)
    tbetr = Column(String)
    fikey = Column(String)
    bvrko = Column(String)
    bukrs = Column(String)
    gsber = Column(String)
    blart = Column(String)
    waers = Column(String)
    kursf = Column(String)
    budat = Column(String)
    bldat = Column(String)
    valut = Column(String)
    xeiph = Column(String)
    augrd = Column(String)
    aenam = Column(String)
    aedat = Column(String)
    aetim = Column(String)
    xrues = Column(String)
    xakon = Column(String)
    xklae = Column(String)
    klaeb = Column(String)
    klaeh = Column(String)
    klaed = Column(String)
    opbel = Column(String)
    uposz = Column(String)
    uposv = Column(String)
    txtvw = Column(String)
    chckn = Column(String)
    banks = Column(String)
    bankl = Column(String)
    bankn = Column(String)
    bkont = Column(String)
    koinh = Column(String)
    xpgro = Column(String)
    xdaub = Column(String)
    nrzaa = Column(String)
    repym = Column(String)
    xclar = Column(String)
    infof = Column(String)
    kukon = Column(String)
    ccins = Column(String)
    ccnum = Column(String)
    ruebl = Column(String)
    ruear = Column(String)
    bkref = Column(String)
    txtrz = Column(String)
    hzuon = Column(String)
    datab = Column(String)
    datbi = Column(String)
    aunum = Column(String)
    audat = Column(String)
    autim = Column(String)
    klaec = Column(String)
    xatts = Column(String)
    cguid = Column(String)
    enctp = Column(String)
    merch = Column(String)


class Dfkkzk(Base):
    __tablename__ = 'DFKKZK'

    mandt = Column(String, primary_key=True)
    keyz1 = Column(String)
    keyz2 = Column(String)
    applk = Column(String)
    herkf = Column(String)
    fikey = Column(String)
    bvrko = Column(String)
    bukrs = Column(String)
    gsber = Column(String)
    blart = Column(String)
    waers = Column(String)
    kursf = Column(String)
    budat = Column(String)
    bldat = Column(String)
    valut = Column(String)
    xeiph = Column(String)
    augrd = Column(String)
    selt1 = Column(String)
    selt2 = Column(String)
    selt3 = Column(String)
    anzpo = Column(String)
    summs = Column(String)
    summh = Column(String)
    sumwa = Column(String)
    ernam = Column(String)
    erdat = Column(String)
    ertim = Column(String)
    aenam = Column(String)
    aedat = Column(String)
    aetim = Column(String)
    maxup = Column(String)
    maxuv = Column(String)
    stazs = Column(String)
    xschs = Column(String)
    xcrds = Column(String)
    xzaus = Column(String)
    cczah = Column(String)
    subel = Column(String)
    vkoch = Column(String)
    xebok = Column(String)
    xposa = Column(String)
    infof = Column(String)
    ktsus = Column(String)
    ktsuh = Column(String)
    ksump = Column(String)
    xnseb = Column(String)
    jobname = Column(String)
    chdsk = Column(String)
    paytp = Column(String)
    xauts = Column(String)
    cvscd = Column(String)
    xexpy = Column(String)

class Dfkkopk(Base):
    __tablename__ = 'DFKKOPK'

    mandt = Column(String, primary_key=True)
    opbel = Column(String)
    valut = Column(String)
    fikey = Column(String)
    hkont = Column(String)
    betrh = Column(String)
    intbu = Column(String)
    gsber = Column(String)


class Dfkkopkc(Base):
    __tablename__ = 'DFKKOPKC'

    mandt = Column(String, primary_key=True)
    opbel = Column(String)
    opupk = Column(String)
    ccins = Column(String)
    ccnum = Column(String)
    ccfol = Column(String)
    datab = Column(String)
    datbi = Column(String)
    ccname = Column(String)
    aunum = Column(String)
    autra = Column(String)
    audat = Column(String)
    autim = Column(String)
    auart = Column(String)
    autwr = Column(String)
    ccwae = Column(String)
    merch = Column(String)
    budat = Column(String)
    betrw = Column(String)
    waers = Column(String)
    bukrs = Column(String)
    xsett = Column(String)
    laufi = Column(String)
    stdoc = Column(String)
    laufd_pyp = Column(String)
    laufi_pyp = Column(String)
    orign_pyp = Column(String)
    anzrate = Column(String)
    cvval = Column(String)
    cvvct = Column(String)
    cvvst = Column(String)
    zzacuenta = Column(String)
    valut = Column(String)
    cguid = Column(String)
    enctp = Column(String)
    resend = Column(String)
    xguid = Column(String)


class Dfkksum(Base):
    __tablename__ = 'DFKKSUM'

    mandt = Column(String, primary_key=True)
    fikey = Column(String)
    shkzg = Column(String)
    hkont = Column(String)
    gsber = Column(String)
    budat = Column(String)
    bldat = Column(String)
    valut = Column(String)
    betrh = Column(String)


class Dfkkko(Base):
    __tablename__ = 'DFKKKO'

    mandt = Column(String, primary_key=True)
    opbel = Column(String)
    blart = Column(String)
    budat = Column(String)
    bldat = Column(String)
    storb = Column(String)
    ernam = Column(String)
    cpudt = Column(String)
    cputm = Column(String)
    fikey = Column(String)


class Dfkkop(Base):
    __tablename__ = 'DFKKOP'

    mandt = Column(String, primary_key=True)
    opbel = Column(String)
    gsber = Column(String)
    augst = Column(String)
    vkont = Column(String)
    faedn = Column(String)
    bldat = Column(String)
    betrh = Column(String)
    augbl = Column(String)
    augdt = Column(String)
    augbt = Column(String)
    augob = Column(String)
    blart = Column(String)

